@@ -1,11 +1,23 @@
 /*
  * catman.c: create and/or update cat files
  *  
- * Copyright (C), 1994, 1995, Graeme W. Wilford. (Wilf.)
+ * Copyright (C) 1994, 1995 Graeme W. Wilford. (Wilf.)
  *
- * You may distribute under the terms of the GNU General Public
- * License as specified in the file COPYING that comes with this
- * distribution.
+ * This file is part of man-db.
+ *
+ * man-db is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * man-db is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with man-db; if not, write to the Free Software Foundation,
+ * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Thu Dec  8 00:03:12 GMT 1994  Wilf. (G.Wilford@ee.surrey.ac.uk) 
  */
@@ -119,11 +131,11 @@ static void usage (int status)
 	printf (_("usage: %s [-dhV] [-M manpath] [section] ...\n"),
 		program_name);
 	printf (_(
-		"-d --debug                  produce debugging info.\n"
-		"-M --manpath path           set search path for manual pages "
+		"-d, --debug                 produce debugging info.\n"
+		"-M, --manpath path          set search path for manual pages "
 					    "to `path'.\n"
-		"-V --version                show version.\n"
-		"-h --help                   show this usage message.\n"));
+		"-V, --version               show version.\n"
+		"-h, --help                  show this usage message.\n"));
 
 	exit (status);
 }
