@@ -1,11 +1,24 @@
 /*
  * compression.c: code to find decompressor / compression extension
  *  
- * Copyright (C), 1994, 1995, Graeme W. Wilford. (Wilf.)
+ * Copyright (C) 1994, 1995 Graeme W. Wilford. (Wilf.)
+ * Copyright (C) 2001, 2002 Colin Watson.
  *
- * You may distribute under the terms of the GNU General Public
- * License as specified in the file COPYING that comes with this
- * distribution.
+ * This file is part of man-db.
+ *
+ * man-db is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * man-db is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with man-db; if not, write to the Free Software Foundation,
+ * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Sat Aug 20 15:01:02 BST 1994  Wilf. (G.Wilford@ee.surrey.ac.uk) 
  */
@@ -136,7 +149,10 @@ struct compression *comp_file (const char *filename)
 	return NULL;
 }
 
-/* set up a pointer to a unique temp filename on first call */
+/* Set up a pointer to a unique temp filename on first call.
+ * If this returns NULL, an error message will have been printed and the
+ * caller should abort the current operation as appropriate.
+ */
 char *decompress (const char *filename, const struct compression *comp)
 {
 	char *command;
@@ -171,7 +187,9 @@ char *decompress (const char *filename, const struct compression *comp)
 
 	if (status) {
 		remove_ztemp ();
-		exit (CHILD_FAIL);
+		error (0, 0, _("command '%s' failed with exit status %d"),
+		       command, status);
+		return NULL;
 	}
 	return file;
 }
