@@ -1,12 +1,25 @@
 /*
- * manp.c
+ * manp.c: Manpath calculations
  *
- * Copyright (c) 1990, 1991, John W. Eaton.
- * Copyright (C) 1994, 1995, Graeme W. Wilford. (Wilf.)
+ * Copyright (C) 1990, 1991 John W. Eaton.
+ * Copyright (C) 1994, 1995 Graeme W. Wilford. (Wilf.)
+ * Copyright (C) 2001, 2002 Colin Watson.
  *
- * You may distribute under the terms of the GNU General Public
- * License as specified in the file COPYING that comes with this
- * distribution.
+ * This file is part of man-db.
+ *
+ * man-db is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * man-db is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with man-db; if not, write to the Free Software Foundation,
+ * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * John W. Eaton
  * jwe@che.utexas.edu
@@ -471,9 +484,10 @@ static char *add_system_manpath (const char *systems, const char *manpathlist)
 				char *element;
 
 				next = strchr (path, ':');
-				if (next)
+				if (next) {
 					element = xstrndup (path, next - path);
-				else
+					++next;
+				} else
 					element = xstrdup (path);
 				newdir = strappend (newdir, element, "/",
 						    system, NULL);
