@@ -1,11 +1,24 @@
 /*
  * manpath.c: display either the manpath or catpath
  *  
- * Copyright (C), 1994, 1995, Graeme W. Wilford. (Wilf.)
+ * Copyright (C) 1994, 1995 Graeme W. Wilford. (Wilf.)
+ * Copyright (C) 2001, 2002 Colin Watson.
  *
- * You may distribute under the terms of the GNU General Public
- * License as specified in the file COPYING that comes with this
- * distribution.
+ * This file is part of man-db.
+ *
+ * man-db is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * man-db is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with man-db; if not, write to the Free Software Foundation,
+ * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Thu Nov 17 08:29:39 GMT 1994  Wilf. (G.Wilford@ee.surrey.ac.uk) 
  */
@@ -60,9 +73,7 @@ extern pid_t vfork();
 extern char *optarg;
 extern int optind, opterr, optopt;
 
-#ifndef debug
-int debug;
-#endif
+int debug = 0;
 char *program_name;
 int quiet = 0;
 
@@ -80,18 +91,19 @@ static const struct option long_options[] =
 
 static const char args[] = "cgdhqVm:";
 
-static void usage(int status)
+static void usage (int status)
 {
 	printf (_( 
-		"usage: %s [[-gcdq] [-m system]] | [-V] | [-h]\n"), program_name);
+		"usage: %s [[-gcdq] [-m system]] | [-V] | [-h]\n"),
+		program_name);
 	printf (_(
-		"-c --catpath                show relative catpaths.\n"
-		"-g --global                 show the entire global manpath.\n"
-	        "-d --debug                  produce debugging info.\n"
-	        "-q --quiet                  produce fewer warnings.\n"
-	        "-m --systems system         express which `systems' to use.\n"
-	        "-V --version                show version.\n"
-	        "-h --help                   show this usage message.\n"));
+		"-c, --catpath               show relative catpaths.\n"
+		"-g, --global                show the entire global manpath.\n"
+	        "-d, --debug                 produce debugging info.\n"
+	        "-q, --quiet                 produce fewer warnings.\n"
+	        "-m, --systems system        express which `systems' to use.\n"
+	        "-V, --version               show version.\n"
+	        "-h, --help                  show this usage message.\n"));
 
 	exit (status);
 }
@@ -107,18 +119,19 @@ int main (int argc, char *argv[])
 	int option_index; /* not used, but required by getopt_long() */
 	char *locale;
 
-	program_name = xstrdup(basename((argv[0])));
+	program_name = xstrdup (basename (argv[0]));
 	/* initialise the locale */
-	locale = setlocale( LC_MESSAGES, "");
+	locale = setlocale (LC_ALL, "");
+	if (locale)
+		locale = xstrdup (locale);
+	else {
+		/* Obviously can't translate this. */
+		error (0, 0, "can't set the locale; make sure $LC_* and $LANG "
+			     "are correct");
+		locale = "C";
+	}
 	bindtextdomain (PACKAGE, LOCALEDIR);
 	textdomain (PACKAGE);
-	if (locale != NULL
-	|| (locale = getenv ("LC_ALL"))
-	|| (locale = getenv ("LC_MESSAGES"))
-	|| (locale = getenv ("LANG")) )
-		locale = xstrdup (locale);
-	else
-		locale = "C";
 
 	while ((c = getopt_long (argc, argv, args,
 				 long_options, &option_index)) != EOF) {
@@ -128,9 +141,7 @@ int main (int argc, char *argv[])
 				cat = 1;
 				break;
 			case 'd':
-#ifndef debug
 			    	debug = 1;
-#endif
 			    	break;
 		    	case 'q':
 			    	quiet = 1;
@@ -157,12 +168,15 @@ int main (int argc, char *argv[])
 	path_string = manpath (alt_system);
 
 	if (global) {
-		path_string = get_mandb_manpath();
+		path_string = get_mandb_manpath ();
 		if (!path_string)
-			error (FAIL, 0, _( "warning: no global manpaths set in config file %s"), CONFIG_FILE);
+			error (FAIL, 0,
+			       _("warning: no global manpaths set in "
+				 "config file %s"),
+			       CONFIG_FILE);
 	}
 	if (cat)
-		path_string = cat_manpath(path_string);
+		path_string = cat_manpath (path_string);
 
 	printf ("%s\n", path_string);
 	exit (OK);
